package de.twomartens.oparlservice.control;

import de.twomartens.oparlservice.entity.*;
import de.twomartens.oparlservice.entity.System;
import de.twomartens.oparlservice.service.OParlService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Slf4j
@RestController
@RequestMapping(path = "/v1.1")
public class OParlController {
    private final OParlService service;

    OParlController(OParlService service) {
        this.service = service;
    }

    @GetMapping("/")
    @Operation(summary = "System information", description = "returns information about the OParl system")
    public System system() {
        log.info("method invoked /v1.1");
        return service.getSystem();
    }

    @GetMapping("/bodies")
    @Operation(summary = "List of available bodies", description = "returns a list of available bodies in this OParl system")
    public List<Body> bodies() {
        log.info("method invoked /v1.1/bodies");
        return this.service.getBodies();
    }

    @GetMapping("/body/{id}")
    @Operation(summary = "information about body", description = "returns information about the requested body")
    public Body body(
            @PathVariable
            @Parameter(description = "body ID", example = "0")
                    String id) {
        log.info("method invoked /v1.1/body/{}", id);
        return service.getBody(id);
    }

    @GetMapping("/term/{id}")
    @Operation(summary = "information about legislative term", description = "returns information about the requested legislative term")
    public LegislativeTerm legislativeTerm(
            @PathVariable
            @Parameter(description = "legislative term ID", example = "21")
                    String id) {
        log.info("method invoked /v1.1/term/{}", id);
        return service.getLegislativeTerm(id);
    }

    @GetMapping("/body/{id}/organizations")
    @Operation(summary = "List of all organizations in body", description = "returns a list of all organizations in requested body")
    public List<Organization> organizationsInBody(
            @PathVariable
            @Parameter(description = "body ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/body/{}/organizations", id);
        return service.getOrganizationsInBody(id);
    }

    @GetMapping("/organization/{id}")
    @Operation(summary = "information about organization", description = "returns the requested organization")
    public Organization organization(
            @PathVariable
            @Parameter(description = "organization ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/organization/{}", id);
        return service.getOrganization(id);
    }

    @GetMapping("/body/{id}/persons")
    @Operation(summary = "List of all persons in body", description = "returns a list of all persons in requested body")
    public List<Person> persons(
            @PathVariable
            @Parameter(description = "body ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/body/{}/persons", id);
        return service.getPersonsInBody(id);
    }

    @GetMapping("/person/{id}")
    @Operation(summary = "information about person", description = "returns the requested person")
    public Person person(
            @PathVariable
            @Parameter(description = "person ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/person/{}", id);
        return service.getPerson(id);
    }

    @GetMapping("/body/{id}/memberships")
    @Operation(summary = "List of all memberships in body", description = "returns a list of all memberships in requested body")
    public List<Membership> membershipsInBody(
            @PathVariable
            @Parameter(description = "body ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/body/{}/memberships", id);
        return service.getMembershipsInBody(id);
    }

    @GetMapping("/organization/{id}/memberships")
    @Operation(summary = "List of all memberships in organization", description = "returns a list of all memberships in requested organization")
    public List<Membership> membershipsInOrganization(
            @PathVariable
            @Parameter(description = "organization ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/organization/{}/memberships", id);
        return service.getMembershipsInOrganization(id);
    }

    @GetMapping("/membership/{id}")
    @Operation(summary = "information about membership", description = "returns the requested membership")
    public Membership membership(
            @PathVariable
            @Parameter(description = "membership ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/membership/{}", id);
        return service.getMembership(id);
    }

    @GetMapping("/body/{id}/meetings")
    @Operation(summary = "List of all meetings in body", description = "returns a list of all meetings in requested body")
    public List<Meeting> meetingsInBody(
            @PathVariable
            @Parameter(description = "body ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/body/{}/meetings", id);
        return service.getMeetingsInBody(id);
    }

    @GetMapping("/organization/{id}/meetings")
    @Operation(summary = "List of all meetings in organization", description = "returns a list of all meetings in requested organization")
    public List<Meeting> meetingsInOrganization(
            @PathVariable
            @Parameter(description = "organization ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/organization/{}/meetings", id);
        return service.getMeetingsInOrganization(id);
    }

    @GetMapping("/meeting/{id}")
    @Operation(summary = "information about meeting", description = "returns the requested meeting")
    public Meeting meeting(
            @PathVariable
            @Parameter(description = "meeting ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/meeting/{}", id);
        return service.getMeeting(id);
    }

    @GetMapping("/agendaItem/{id}")
    @Operation(summary = "information about agenda item", description = "returns the requested agenda item")
    public AgendaItem agendaItem(
            @PathVariable
            @Parameter(description = "agendaItem ID", example = "0")
                    String id) {
        log.info("invoked method /v1.1/agendaItem/{}", id);
        return service.getAgendaItem(id);
    }
}
